@@ -12,8 +12,12 @@ const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 const ExpoThumbnail = imports.ui.expoThumbnail;
 
+// ***************
+// This shows all of the workspaces
+// ***************
+
 // Time for initial animation going into Overview mode
-const ANIMATION_TIME = 0.2;
+const ANIMATION_TIME = 200;
 
 function Expo() {
     this._init.apply(this, arguments);
@@ -34,7 +38,12 @@ Expo.prototype = {
         // one. Instances of this class share a single CoglTexture behind the
         // scenes which allows us to show the background with different
         // rendering options without duplicating the texture data.
-        this._background = Meta.BackgroundActor.new_for_screen(global.screen);
+        if (!Meta.is_wayland_compositor()) {
+            this._background = Meta.X11BackgroundActor.new_for_display(global.display);
+        } else {
+            this._background = new Clutter.Actor();
+        }
+
         this._background.hide();
         global.overlay_group.add_actor(this._background);
 
@@ -232,17 +241,20 @@ Expo.prototype = {
     _showCloseArea : function() {
         let primary = Main.layoutManager.primaryMonitor;
         this._windowCloseArea.show();
-        Tweener.addTween(this._windowCloseArea, {   y: primary.height - this._windowCloseArea.height,
-                                                    time: ANIMATION_TIME,
-                                                    transition: 'easeOutQuad'});
+        this._windowCloseArea.ease({
+            y: primary.height - this._windowCloseArea.height,
+            duration: Main.animations_enabled ? ANIMATION_TIME : 0,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD
+        });
     },
 
     _hideCloseArea : function() {
         let primary = Main.layoutManager.primaryMonitor;
-        Tweener.addTween(this._windowCloseArea, {   y: primary.height,
-                                                    time: ANIMATION_TIME,
-                                                    transition: 'easeOutQuad',
-                                                    onComplete: this.hide});
+        this._windowCloseArea.ease({
+            y: primary.height,
+            duration: Main.animations_enabled ? ANIMATION_TIME : 0,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD
+        });
     },
 
     //// Public methods ////
@@ -279,7 +291,7 @@ Expo.prototype = {
         // clones of them, this would obviously no longer be necessary.
         //
         // Disable unredirection while in the overview
-        Meta.disable_unredirect_for_screen(global.screen);
+        Meta.disable_unredirect_for_display(global.display);
         global.window_group.hide();
         this._group.show();
         this._background.show();
@@ -292,60 +304,54 @@ Expo.prototype = {
         let activeWorkspace = this._expo.lastActiveWorkspace;
         let activeWorkspaceActor = activeWorkspace.actor;
 
-        // should not create new actors and work with them within an allocation cycle
-        let clones = [];
-        Main.layoutManager.monitors.forEach(function(monitor,index) {
-            let clone = new Clutter.Clone({source: activeWorkspaceActor});
-            global.overlay_group.add_actor(clone);
-            clone.set_clip(monitor.x, monitor.y, monitor.width, monitor.height);
-            clones.push(clone);
-        }, this);
-        let animate = Main.wm.settingsState['desktop-effects-workspace'];
         //We need to allocate activeWorkspace before we begin its clone animation
         let allocateID = this._expo.connect('allocated', Lang.bind(this, function() {
             this._expo.disconnect(allocateID);
+
+            let clones = [];
+            Main.layoutManager.monitors.forEach(function(monitor,index) {
+                let clone = new Clutter.Clone({source: activeWorkspaceActor});
+                global.overlay_group.add_actor(clone);
+                clone.set_clip(monitor.x, monitor.y, monitor.width, monitor.height);
+                clones.push(clone);
+            }, this);
+
             Main.layoutManager.monitors.forEach(function(monitor,index) {
                 let clone = clones[index];
-                if (animate) {
-                    Tweener.addTween(clone, {
-                        x: Main.layoutManager.primaryMonitor.x + activeWorkspaceActor.allocation.x1,
-                        y: Main.layoutManager.primaryMonitor.y + activeWorkspaceActor.allocation.y1,
-                        scale_x: activeWorkspaceActor.get_scale()[0] , 
-                        scale_y: activeWorkspaceActor.get_scale()[1], 
-                        time: ANIMATION_TIME,
-                        transition: 'easeOutQuad', 
-                        onComplete: function() {
-                            global.overlay_group.remove_actor(clone);
-                            clone.destroy();
-                            if (index == Main.layoutManager.monitors.length < 1) {
-                                this._showDone();
-                            }
-                        }, 
-                        onCompleteScope: this
-                    });
-                }
-                else {
-                    global.overlay_group.remove_actor(clone);
-                    clone.destroy();
-                    if (index == Main.layoutManager.monitors.length < 1) {
-                        this._showDone();
+                clone.ease({
+                    x: Main.layoutManager.primaryMonitor.x + activeWorkspaceActor.allocation.x1,
+                    y: Main.layoutManager.primaryMonitor.y + activeWorkspaceActor.allocation.y1,
+                    scale_x: activeWorkspaceActor.get_scale()[0] , 
+                    scale_y: activeWorkspaceActor.get_scale()[1], 
+                    duration: Main.animations_enabled ? ANIMATION_TIME : 0,
+                    mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                    onUpdate: (t, timeIndex) => {
+                        clone.get_transition("x")?.set_to(Main.layoutManager.primaryMonitor.x + activeWorkspaceActor.allocation.x1);
+                        clone.get_transition("y")?.set_to(Main.layoutManager.primaryMonitor.y + activeWorkspaceActor.allocation.y1);
+                        clone.get_transition("scale-x")?.set_to(activeWorkspaceActor.get_scale()[0]);
+                        clone.get_transition("scale-y")?.set_to(activeWorkspaceActor.get_scale()[1]);
+                    },
+                    onComplete: () => {
+                        global.overlay_group.remove_actor(clone);
+                        clone.destroy();
+                        if (index == Main.layoutManager.monitors.length - 1) {
+                            this._showDone();
+                        }
                     }
-                }
+                });
             }, this);
         }));
         this._gradient.show();
         Main.panelManager.disablePanels();
 
-        if (animate) {
-            this._background.dim_factor = 1;
-            Tweener.addTween(this._background,
-                            { dim_factor: 0.4,
-                              transition: 'easeOutQuad',
-                              time: ANIMATION_TIME});
-        }
-        else {
-            this._background.dim_factor = 0.4;
-        }
+        this._background.dim_factor = 1;
+        this._background.ease({
+            dim_factor: 0.4,
+            duration: Main.animations_enabled ? ANIMATION_TIME : 0,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD
+        });
+
+        activeWorkspace.setOverviewMode(true);
 
         this._coverPane.raise_top();
         this._coverPane.show();
@@ -389,7 +395,8 @@ Expo.prototype = {
             }
         } else {
             if (this._modal) {
-                Main.popModal(this._group);
+                if (this._group != null)
+                    Main.popModal(this._group);
                 this._modal = false;
             }
             else if (global.stage_input_mode == Cinnamon.StageInputMode.FULLSCREEN)
@@ -401,9 +408,6 @@ Expo.prototype = {
         if (!this.visible || this.animationInProgress)
             return;
 
-        let animationTime = ANIMATION_TIME;
-        this.animationInProgress = true;
-        this._hideInProgress = true;
 
         let activeWorkspace = this._expo.lastActiveWorkspace;
 
@@ -412,6 +416,17 @@ Expo.prototype = {
             activeWorkspace.overviewModeOff(true, true);
         }
 
+        let animate = Main.animations_enabled;
+
+        if (!animate) {
+            this._group.hide();
+            this._hideDone();
+            return;
+        }
+
+        this.animationInProgress = true;
+        this._hideInProgress = true;
+
         let activeWorkspaceActor = activeWorkspace.actor;
         Main.layoutManager.monitors.forEach(function(monitor,index) {
             let cover = new Clutter.Group();
@@ -425,34 +440,22 @@ Expo.prototype = {
             clone.set_clip(monitor.x, monitor.y, monitor.width, monitor.height);
             clone.set_scale(activeWorkspaceActor.get_scale()[0], activeWorkspaceActor.get_scale()[1]);
 
-            let animate = Main.wm.settingsState['desktop-effects-workspace'];
-            if (animate) {
-                Tweener.addTween(clone, {
-                    x: 0,
-                    y: 0,
-                    scale_x: 1,
-                    scale_y: 1,
-                    time: animationTime,
-                    transition: 'easeOutQuad',
-                    onCompleteScope: this,
-                    onComplete: function() {
-                        global.overlay_group.remove_actor(cover);
-                        cover.destroy();
-                        if (index == Main.layoutManager.monitors.length < 1) {
-                            this._group.hide();
-                            this._hideDone();
-                        }
+            clone.ease({
+                x: 0,
+                y: 0,
+                scale_x: 1,
+                scale_y: 1,
+                duration: Main.animations_enabled ? ANIMATION_TIME : 0,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onComplete: () => {
+                    global.overlay_group.remove_actor(cover);
+                    cover.destroy();
+                    if (index == Main.layoutManager.monitors.length - 1) {
+                        this._group.hide();
+                        this._hideDone();
                     }
-                });
-            }
-            else {
-                global.overlay_group.remove_actor(cover);
-                cover.destroy();
-                if (index == Main.layoutManager.monitors.length < 1) {
-                    this._group.hide();
-                    this._hideDone();
                 }
-            }
+            });
         }, this);
 
         this.emit('hiding');
@@ -473,7 +476,7 @@ Expo.prototype = {
 
     _hideDone: function() {
         // Re-enable unredirection
-        Meta.enable_unredirect_for_screen(global.screen);
+        Meta.enable_unredirect_for_display(global.display);
 
         global.window_group.show();
 
