@@ -25,6 +25,7 @@
  * @statusIconDispatcher (StatusIconDispatcher.StatusIconDispatcher): The status icon dispatcher
  * @virtualKeyboard (VirtualKeyboard.Keyboard): The keyboard object
  * @layoutManager (Layout.LayoutManager): The layout manager.
+ * @monitorLabeler (MonitorLabeler.MonitorLabeler): Adds labels to each monitor when configuring displays.
  * \
  * All actors that are part of the Cinnamon UI ar handled by the layout
  * manager, which will determine when to show and hide the actors etc.
@@ -53,6 +54,7 @@
  *
  * @keybindingManager (KeybindingManager.KeybindingManager): The keybinding manager
  * @systrayManager (Systray.SystrayManager): The systray manager
+ * @gesturesManager (GesturesManager.GesturesManager): Gesture support  from ToucheEgg.
  *
  * @osdWindow (OsdWindow.OsdWindow): Osd window that pops up when you use media
  * keys.
@@ -86,8 +88,10 @@ const GObject = imports.gi.GObject;
 const XApp = imports.gi.XApp;
 const PointerTracker = imports.misc.pointerTracker;
 
+const AudioDeviceSelection = imports.ui.audioDeviceSelection;
 const SoundManager = imports.ui.soundManager;
 const BackgroundManager = imports.ui.backgroundManager;
+const Config = imports.misc.config;
 const SlideshowManager = imports.ui.slideshowManager;
 var AppletManager = imports.ui.appletManager;
 const SearchProviderManager = imports.ui.searchProviderManager;
@@ -100,9 +104,12 @@ const Overview = imports.ui.overview;
 const Expo = imports.ui.expo;
 const Panel = imports.ui.panel;
 const PlacesManager = imports.ui.placesManager;
+const PolkitAuthenticationAgent = imports.ui.polkitAuthenticationAgent;
+const KeyringPrompt = imports.ui.keyringPrompt;
 const RunDialog = imports.ui.runDialog;
 const Layout = imports.ui.layout;
 const LookingGlass = imports.ui.lookingGlass;
+const NetworkAgent = imports.ui.networkAgent;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const CinnamonDBus = imports.ui.cinnamonDBus;
@@ -118,10 +125,12 @@ const Settings = imports.ui.settings;
 const Systray = imports.ui.systray;
 const Accessibility = imports.ui.accessibility;
 const ModalDialog = imports.ui.modalDialog;
-const {readOnlyError} = imports.ui.environment;
-const {installPolyfills} = imports.ui.overrides;
 const InputMethod = imports.misc.inputMethod;
 const ScreenRecorder = imports.ui.screenRecorder;
+const {GesturesManager} = imports.ui.gestures.gesturesManager;
+const {MonitorLabeler} = imports.ui.monitorLabeler;
+const {CinnamonPortalHandler} = imports.misc.portalHandlers;
+const {EndSessionDialog} = imports.ui.endSessionDialog;;
 
 var LAYOUT_TRADITIONAL = "traditional";
 var LAYOUT_FLIPPED = "flipped";
@@ -140,12 +149,14 @@ var overview = null;
 var expo = null;
 var runDialog = null;
 var lookingGlass = null;
+var lookingGlassUpdateID = 0;
 var wm = null;
 var a11yHandler = null;
 var messageTray = null;
 var notificationDaemon = null;
 var windowAttentionHandler = null;
 var screenRecorder = null;
+var cinnamonAudioSelectionDBusService = null;
 var cinnamonDBusService = null;
 var screenshotService = null;
 var modalCount = 0;
@@ -157,6 +168,8 @@ var xdndHandler = null;
 var statusIconDispatcher = null;
 var virtualKeyboard = null;
 var layoutManager = null;
+var networkAgent = null;
+var monitorLabeler = null;
 var themeManager = null;
 var keybindingManager = null;
 var _errorLogStack = [];
@@ -168,7 +181,8 @@ var tracker = null;
 var settingsManager = null;
 var systrayManager = null;
 var wmSettings = null;
-
+var pointerSwitcher = null;
+var gesturesManager = null;
 var workspace_names = [];
 
 var applet_side = St.Side.TOP; // Kept to maintain compatibility. Doesn't seem to be used anywhere
@@ -181,6 +195,9 @@ var popup_rendering_actor = null;
 
 var xlet_startup_error = false;
 
+var endSessionDialog = null;
+
+var gpuOffloadHelper = null;
 var gpu_offload_supported = false;
 
 var RunState = {
@@ -227,7 +244,7 @@ function _addXletDirectoriesToSearchPath() {
 }
 
 function _initUserSession() {
-    global.screen.override_workspace_layout(Meta.DisplayCorner.TOPLEFT, false, 1, -1);
+    global.workspace_manager.override_workspace_layout(Meta.DisplayCorner.TOPLEFT, false, 1, -1);
 
     systrayManager = new Systray.SystrayManager();
 
@@ -266,15 +283,13 @@ function start() {
     global.logError = _logError;
     global.log = _logInfo;
 
-    installPolyfills(readOnlyError, _log);
-
     let cinnamonStartTime = new Date().getTime();
 
-    log("About to start Cinnamon");
+    log(`About to start Cinnamon (${Meta.is_wayland_compositor() ? "Wayland" : "X11"} backend)`);
 
     let backend = Meta.get_backend();
 
-    // Only cinnamon2d laucher will set CINNAMON_2D - this is deliberate by the user.
+    // Only cinnamon2d launcher will set CINNAMON_2D - this is deliberate by the user.
     let cinnamon_2d = GLib.getenv("CINNAMON_2D") === true;
     let live = false;
 
@@ -301,6 +316,8 @@ function start() {
 
     Clutter.get_default_backend().set_input_method(new InputMethod.InputMethod());
 
+    new CinnamonPortalHandler();
+    cinnamonAudioSelectionDBusService = new AudioDeviceSelection.AudioDeviceSelectionDBus();
     cinnamonDBusService = new CinnamonDBus.CinnamonDBus();
     setRunState(RunState.STARTUP);
 
@@ -337,16 +354,18 @@ function start() {
     settingsManager = new Settings.SettingsManager();
 
     backgroundManager = new BackgroundManager.BackgroundManager();
+    backgroundManager.hideBackground();
 
     slideshowManager = new SlideshowManager.SlideshowManager();
 
     keybindingManager = new Keybindings.KeybindingManager();
     deskletContainer = new DeskletManager.DeskletContainer();
 
+    gesturesManager = new GesturesManager();
+
     uiGroup = new Layout.UiActor({ name: 'uiGroup' });
     uiGroup.set_flags(Clutter.ActorFlags.NO_LAYOUT);
 
-    global.background_actor.hide();
     global.reparentActor(global.window_group, uiGroup);
     global.reparentActor(global.overlay_group, uiGroup);
 
@@ -361,7 +380,7 @@ function start() {
 
     global.reparentActor(global.top_window_group, global.stage);
 
-    global.menuStackLength = 0;
+    global.menuStack = [];
 
     layoutManager = new Layout.LayoutManager();
 
@@ -376,6 +395,7 @@ function start() {
                                 !software_rendering;
 
     if (do_startup_animation) {
+        backgroundManager.showBackground();
         layoutManager._prepareStartupAnimation();
     }
 
@@ -383,6 +403,14 @@ function start() {
     pointerTracker.setPosition(layoutManager.primaryMonitor.x + layoutManager.primaryMonitor.width/2,
         layoutManager.primaryMonitor.y + layoutManager.primaryMonitor.height/2);
 
+    pointerSwitcher = new PointerTracker.PointerSwitcher();
+
+    if (Meta.is_wayland_compositor()) {
+        monitorLabeler = new MonitorLabeler();
+    } else {
+        monitorLabeler = null;
+    }
+
     xdndHandler = new XdndHandler.XdndHandler();
     osdWindowManager = new OsdWindow.OsdWindowManager();
     // This overview object is just a stub for non-user sessions
@@ -400,6 +428,33 @@ function start() {
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
     placesManager = new PlacesManager.PlacesManager();
 
+    // NM Agent
+    if (Config.HAVE_NETWORKMANAGER && global.settings.get_boolean("enable-nm-agent")) {
+        networkAgent = new NetworkAgent.NetworkAgent();
+        global.log('NetworkManager agent: enabled')
+    }
+    else {
+        global.log('NetworkManager agent: disabled')
+    }
+
+    // Polkit Agent
+    if (global.settings.get_boolean("enable-polkit-agent")) {
+        PolkitAuthenticationAgent.init();
+        global.log('Polkit agent: enabled')
+    }
+    else {
+        global.log('Polkit agent: disabled')
+    }
+
+    // SSH Agent
+    if (global.settings.get_boolean("enable-ssh-agent")) {
+        KeyringPrompt.init();
+        global.log('SSH agent: enabled')
+    }
+    else {
+        global.log('SSH agent: disabled')
+    }
+
     magnifier = new Magnifier.Magnifier();
     locatePointer = new LocatePointer.locatePointer();
 
@@ -412,10 +467,6 @@ function start() {
     _initUserSession();
     screenRecorder = new ScreenRecorder.ScreenRecorder();
 
-    // Provide the bus object for gnome-session to
-    // initiate logouts.
-    //EndSessionDialog.init();
-
     _startDate = new Date();
 
     global.display.connect('restart', () => {
@@ -437,14 +488,11 @@ function start() {
 
     global.display.connect('gl-video-memory-purged', loadTheme);
 
-    try {
-        gpu_offload_supported = Cinnamon.get_gpu_offload_supported()
-    } catch (e) {
-        global.logWarning("Could not check for gpu offload support - maybe xapps isn't up to date.");
-        gpu_offload_supported = false;
-    }
-
-    log(`GPU offload supported: ${gpu_offload_supported}`);
+    gpuOffloadHelper = XApp.GpuOffloadHelper.get();
+    gpuOffloadHelper.connect("ready", (helper, success) => {
+        gpu_offload_supported = success && gpuOffloadHelper.is_offload_supported();
+        global.log(`GPU offload supported: ${gpu_offload_supported}`);
+    });
 
     // We're ready for the session manager to move to the next phase
     GLib.idle_add(GLib.PRIORITY_DEFAULT, () => {
@@ -485,7 +533,7 @@ function start() {
                 return GLib.SOURCE_REMOVE;
             });
         } else {
-            global.background_actor.show();
+            backgroundManager.showBackground();
             setRunState(RunState.RUNNING);
         }
 
@@ -570,7 +618,7 @@ function _fillWorkspaceNames(index) {
 }
 
 function _shouldTrimWorkspace(i) {
-    return i >= 0 && (i >= global.screen.n_workspaces || !workspace_names[i].length);
+    return i >= 0 && (i >= global.workspace_manager.n_workspaces || !workspace_names[i].length);
 }
 
 function _trimWorkspaceNames() {
@@ -636,12 +684,12 @@ function hasDefaultWorkspaceName(index) {
 }
 
 function _addWorkspace() {
-    global.screen.append_new_workspace(false, global.get_current_time());
+    global.workspace_manager.append_new_workspace(false, global.get_current_time());
     return true;
 }
 
 function _removeWorkspace(workspace) {
-    if (global.screen.n_workspaces == 1)
+    if (global.workspace_manager.n_workspaces == 1)
         return false;
     let index = workspace.index();
     if (index < workspace_names.length) {
@@ -649,7 +697,7 @@ function _removeWorkspace(workspace) {
     }
     _trimWorkspaceNames();
     wmSettings.set_strv("workspace-names", workspace_names);
-    global.screen.remove_workspace(workspace, global.get_current_time());
+    global.workspace_manager.remove_workspace(workspace, global.get_current_time());
     return true;
 }
 
@@ -666,16 +714,16 @@ function _removeWorkspace(workspace) {
  */
 function moveWindowToNewWorkspace(metaWindow, switchToNewWorkspace) {
     if (switchToNewWorkspace) {
-        let targetCount = global.screen.n_workspaces + 1;
-        let nnwId = global.screen.connect('notify::n-workspaces', function() {
-            global.screen.disconnect(nnwId);
-            if (global.screen.n_workspaces === targetCount) {
-                let newWs = global.screen.get_workspace_by_index(global.screen.n_workspaces - 1);
+        let targetCount = global.workspace_manager.n_workspaces + 1;
+        let nnwId = global.workspace_manager.connect('notify::n-workspaces', function() {
+            global.workspace_manager.disconnect(nnwId);
+            if (global.workspace_manager.n_workspaces === targetCount) {
+                let newWs = global.workspace_manager.get_workspace_by_index(global.workspace_manager.n_workspaces - 1);
                 newWs.activate(global.get_current_time());
             }
         });
     }
-    metaWindow.change_workspace_by_index(global.screen.n_workspaces, true, global.get_current_time());
+    metaWindow.change_workspace_by_index(global.workspace_manager.n_workspaces, true, global.get_current_time());
 }
 
 /**
@@ -892,8 +940,22 @@ function _log(category = 'info', msg = '') {
 
     _errorLogStack.push(out);
 
-    if (lookingGlass) {
-        lookingGlass.emitLogUpdate();
+    // If the melange window is open/exists, excessive dbus traffic caused by gesture debug
+    // logging can leave the desktop unstable. We end up with lots of:
+    //
+    // Attempting to call back into JSAPI during the sweeping phase of GC...
+    //
+    // This is a hack, and how we handle logging and melange probably needs looked at.
+
+    if (lookingGlass && !gesturesManager.gesture_active()) {
+        if (lookingGlassUpdateID > 0) {
+            GLib.source_remove (lookingGlassUpdateID);
+        }
+
+        lookingGlassUpdateID = GLib.idle_add(GLib.PRIORITY_DEFAULT_IDLE, () => {
+             lookingGlass.emitLogUpdate();
+             lookingGlassUpdateID = 0;
+        });
     }
 
     log(`[LookingGlass/${category}] ${text}`);
@@ -1162,7 +1224,7 @@ function _findModal(actor) {
  * @actor (Clutter.Actor): actor which will be given keyboard focus
  * @timestamp (int): optional timestamp
  * @options (Meta.ModalOptions): (optional) flags to indicate that the pointer
- * is alrady grabbed
+ * is already grabbed
  *
  * Ensure we are in a mode where all keyboard and mouse input goes to
  * the stage, and focus @actor. Multiple calls to this function act in
@@ -1320,13 +1382,13 @@ function getRunDialog() {
  * activation will be handled in muffin.
  */
 function activateWindow(window, time, workspaceNum) {
-    let activeWorkspaceNum = global.screen.get_active_workspace_index();
+    let activeWorkspaceNum = global.workspace_manager.get_active_workspace_index();
 
     if (!time)
         time = global.get_current_time();
 
     if ((workspaceNum !== undefined) && activeWorkspaceNum !== workspaceNum) {
-        let workspace = global.screen.get_workspace_by_index(workspaceNum);
+        let workspace = global.workspace_manager.get_workspace_by_index(workspaceNum);
         workspace.activate_with_focus(window, time);
         return;
     }
@@ -1334,7 +1396,9 @@ function activateWindow(window, time, workspaceNum) {
     window.activate(time);
     Mainloop.idle_add(function() {
         window.foreach_transient(function(win) {
-            win.activate(time);
+            if (win.get_window_type() != Meta.WindowType.NORMAL) {
+                win.activate(time);
+            }
         });
     });
 
@@ -1408,7 +1472,7 @@ function _queueBeforeRedraw(workId) {
  * initialization as well, under the assumption that new actors
  * will need it.
  *
- * Returns (string): A string work identifer
+ * Returns (string): A string work identifier
  */
 function initializeDeferredWork(actor, callback, props) {
     // Turn into a string so we can use as an object property
@@ -1489,8 +1553,7 @@ function isInteresting(metaWindow) {
 
 /**
  * getTabList:
- * @workspaceOpt (Meta.Workspace): (optional) workspace, defaults to global.screen.get_active_workspace()
- * @screenOpt (Meta.Screen): (optional) screen, defaults to global.screen
+ * @workspaceOpt (Meta.Workspace): (optional) workspace, defaults to global.workspace_manager.get_active_workspace()
  *
  * Return a list of the interesting windows on a workspace (by default,
  * the active workspace).
@@ -1498,14 +1561,12 @@ function isInteresting(metaWindow) {
  *
  * Returns (array): list of windows
  */
-function getTabList(workspaceOpt, screenOpt) {
-    let screen = screenOpt || global.screen;
-    let display = screen.get_display();
-    let workspace = workspaceOpt || screen.get_active_workspace();
+function getTabList(workspaceOpt) {
+    let workspace = workspaceOpt || global.workspace_manager.get_active_workspace();
 
     let windows = []; // the array to return
 
-    let allwindows = display.get_tab_list(Meta.TabList.NORMAL_ALL, workspace);
+    let allwindows = global.display.get_tab_list(Meta.TabList.NORMAL_ALL, workspace);
     let registry = {}; // to avoid duplicates
 
     for (let i = 0; i < allwindows.length; ++i) {
@@ -1522,6 +1583,10 @@ function getTabList(workspaceOpt, screenOpt) {
 }
 
 function restartCinnamon(showOsd = false) {
+    if (Meta.is_wayland_compositor()) {
+        global.logWarning("Cinnamon restart not supported with Wayland");
+        return;
+    }
     global.display.connect("show-restart-message", () => {
         if (showOsd) {
             let dialog = new ModalDialog.InfoOSD(_("Restarting Cinnamon..."));
@@ -1535,3 +1600,25 @@ function restartCinnamon(showOsd = false) {
 
     global.reexec_self();
 }
+
+function showEndSessionDialog(mode) {
+    if (endSessionDialog != null) {
+        global.logWarning("End session dialog already exists");
+        return;
+    }
+
+    endSessionDialog = new EndSessionDialog(mode);
+    if (!endSessionDialog.open()) {
+        endSessionDialog = null;
+        global.logWarning("Failed to open end session dialog");
+    }
+}
+
+function closeEndSessionDialog() {
+    if (endSessionDialog == null) {
+        return;
+    }
+
+    endSessionDialog.close();
+    endSessionDialog = null;
+}
